#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/wait.h>
#include <pthread.h>
#include <sys/eventfd.h>

#include "mctd.h"
#include "mctd-log.h"
#include "mctd-atcmd.h"
#include "mctd-modem-control.h"
#include "mctd-thread-queue.h"

static int _efd;
static int _thread_quit = 0;
static int _processing_cmd_id = 0;
pthread_t thread;

void *mctd_thread_modem_control(void *arg)
{
    UNUSED(arg);

    int fd;
    unsigned int cmd_id;
    char *req_param = NULL;
    char *res_param = NULL;
    uint64_t w64 = 1;
    ssize_t wlen;
    int result;

    log_info("modem ctrl thread start.");

    while(!_thread_quit) {
        /* cmd check and function execution */
        if (mctd_thread_queue_req_dequeue(&fd, &cmd_id, &req_param) == MCTD_RESULT_OK) {
            _processing_cmd_id = cmd_id;
            result = mctd_modem_control_exec_app_cmd(cmd_id, req_param, &res_param);
            _processing_cmd_id = 0;

            mctd_thread_queue_res_enqueue(fd, cmd_id, res_param, result);

            do {
                wlen = write(_efd, &w64, sizeof(w64));
            } while (errno == EINTR && wlen < 0);
        }

        free(req_param);
        req_param = NULL;
        free(res_param);
        res_param = NULL;
        pthread_testcancel();
    }

    log_info("modem_ctrl_thread finished.");

    return 0;
}

void mctd_thread_force_quit()
{
    /* modem control thread force quit */
    pthread_cancel(thread);
    pthread_join(thread, NULL);
}

void mctd_thread_set_efd(int efd)
{
    _efd = efd;
}

void mctd_thread_quit()
{
    /* modem control thread successful termination */
    _thread_quit = 1;
    pthread_join(thread, NULL);
}

int mctd_thread_create()
{
    return pthread_create(&thread, NULL, mctd_thread_modem_control, NULL);
}

bool mctd_thread_is_processing()
{
    bool result = false;

    if (_processing_cmd_id)
        result = true;

    return result;
}
